package de.fitko.fitconnect.api.services.crypto;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.exceptions.internal.DecryptionException;
import de.fitko.fitconnect.api.exceptions.internal.EncryptionException;

/**
 * A service that allows to encrypt and decrypt {@link Data} and {@link Attachment}s of a {@link Submission} via JWE.
 *
 * @see <a href="https://datatracker.ietf.org/doc/html/rfc7516">JSON-Web-Encryption</a>
 */
public interface CryptoService {

    /**
     * Decrypts a JWE encrypted string with the given private key
     *
     * @param privateKey RSA private key the encrypted JWE string is decrypted with
     * @param encryptedData serialized encrypted JWE string that is decrypted
     * @return a string serialization of the decrypted payload
     * @throws DecryptionException if the payload cannot be decrypted or there was an issue with the key
     */
    String decryptString(RSAKey privateKey, String encryptedData) throws DecryptionException;

    /**
     * Decrypts a JWE encrypted byte[] with the given private key
     *
     * @param privateKey RSA private key the encrypted JWE string is decrypted with
     * @param encryptedData serialized encrypted JWE string that is decrypted
     * @return a byte[] of the decrypted string payload
     * @throws DecryptionException if the payload cannot be decrypted or there was an issue with the key
     */
    byte[] decryptBytes(RSAKey privateKey, String encryptedData) throws DecryptionException;

    /**
     * JWE encrypts a given string payload with the public key
     *
     * @param publicKey RSA public key the payload is encrypted with
     * @param data json or xml data that should be encrypted
     * @return a string serialization of the encrypted JWE object
     * @throws EncryptionException if the payload cannot be encrypted or there was an issue with the key
     */
    String encryptString(RSAKey publicKey, String data) throws EncryptionException;

    /**
     * JWE encrypts a given byte[] payload with the public key
     *
     * @param publicKey RSA public key the payload is encrypted with
     * @param bytes byte[] of the data that should be encrypted
     * @return a string serialization of the encrypted JWE object
     * @throws EncryptionException if the payload cannot be encrypted or there was an issue with the key
     */
    String encryptBytes(RSAKey publicKey, byte[] bytes) throws EncryptionException;
}
